@@ -5,23 +5,57 @@ use core::fmt;
 
 use k256::ecdsa::recoverable;
 use k256::ecdsa::signature::Signature as SignatureTrait;
-use serde::{Deserialize, Serialize};
+use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
 use sha3::{Digest, Keccak256};
 use umbral_pre::{PublicKey, SerializableToArray, Signature, Signer};
 
 use crate::address::Address;
-use crate::arrays_as_bytes;
+use crate::arrays_as_bytes::{self, DeserializeAsBytes, SerializeAsBytes};
 use crate::fleet_state::FleetStateChecksum;
 use crate::versioning::{
     messagepack_deserialize, messagepack_serialize, ProtocolObject, ProtocolObjectInner,
 };
+use crate::VerificationError;
+
+impl SerializeAsBytes for recoverable::Signature {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_bytes(self.as_ref())
+    }
+}
+
+impl<'de> DeserializeAsBytes<'de> for recoverable::Signature {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        struct BytesVisitor;
+
+        impl<'de> de::Visitor<'de> for BytesVisitor {
+            type Value = recoverable::Signature;
+
+            fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+                write!(f, "Recoverable signature bytes")
+            }
+
+            fn visit_bytes<E>(self, v: &[u8]) -> Result<Self::Value, E>
+            where
+                E: de::Error,
+            {
+                recoverable::Signature::from_bytes(v).map_err(de::Error::custom)
+            }
+        }
+
+        deserializer.deserialize_bytes(BytesVisitor)
+    }
+}
 
 /// Indicates an error during canonical address derivation from a signature.
 pub enum AddressDerivationError {
     /// Signature is missing from the payload.
     NoSignatureInPayload,
-    /// Invalid signature format.
-    InvalidSignature(signature::Error),
     /// Failed to recover the public key from the signature.
     RecoveryFailed(signature::Error),
 }
@@ -30,7 +64,6 @@ impl fmt::Display for AddressDerivationError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::NoSignatureInPayload => write!(f, "Signature is missing from the payload"),
-            Self::InvalidSignature(err) => write!(f, "Invalid signature format: {}", err),
             Self::RecoveryFailed(err) => write!(
                 f,
                 "Failed to recover the public key from the signature: {}",
@@ -56,8 +89,8 @@ pub const RECOVERABLE_SIGNATURE_SIZE: usize = recoverable::SIZE;
 /// Node metadata.
 #[derive(PartialEq, Debug, Serialize, Deserialize, Clone)]
 pub struct NodeMetadataPayload {
-    /// The staker's Ethereum address.
-    pub staker_address: Address,
+    /// The staking provider's Ethereum address.
+    pub staking_provider_address: Address,
     /// The network identifier.
     pub domain: String,
     /// The timestamp of the metadata creation.
@@ -66,16 +99,16 @@ pub struct NodeMetadataPayload {
     pub verifying_key: PublicKey,
     /// The node's encrypting key.
     pub encrypting_key: PublicKey,
-    /// The node's SSL certificate (serialized in PEM format).
+    /// The node's SSL certificate (serialized in DER format).
     #[serde(with = "serde_bytes")]
-    pub certificate_bytes: Box<[u8]>,
+    pub certificate_der: Box<[u8]>,
     /// The hostname of the node's REST service.
     pub host: String,
     /// The port of the node's REST service.
     pub port: u16,
     /// The node's verifying key signed by the private key corresponding to the operator address.
     #[serde(with = "arrays_as_bytes")]
-    pub decentralized_identity_evidence: Option<[u8; RECOVERABLE_SIGNATURE_SIZE]>,
+    pub operator_signature: Option<recoverable::Signature>,
 }
 
 impl NodeMetadataPayload {
@@ -85,13 +118,11 @@ impl NodeMetadataPayload {
     }
 
     /// Derives the address corresponding to the public key that was used
-    /// to create `decentralized_identity_evidence`.
+    /// to create `operator_signature`.
     pub fn derive_operator_address(&self) -> Result<Address, AddressDerivationError> {
-        let evidence = self
-            .decentralized_identity_evidence
+        let signature = self
+            .operator_signature
             .ok_or(AddressDerivationError::NoSignatureInPayload)?;
-        let signature = recoverable::Signature::from_bytes(&evidence)
-            .map_err(AddressDerivationError::InvalidSignature)?;
         let message = encode_defunct(&self.verifying_key.to_array());
         let key = signature
             .recover_verify_key_from_digest(message)
@@ -123,6 +154,8 @@ impl NodeMetadata {
         // This method returns bool and not NodeMetadataPayload,
         // because NodeMetadata can be used before verification,
         // so we need access to its fields right away.
+        // This may change depending on the decision in
+        // https://github.com/nucypher/nucypher/issues/2876
 
         // We could do this on deserialization, but it is a relatively expensive operation.
         self.signature
@@ -243,14 +276,17 @@ impl MetadataResponse {
     }
 
     /// Verifies the metadata response and returns the contained payload.
-    pub fn verify(&self, verifying_pk: &PublicKey) -> Option<MetadataResponsePayload> {
+    pub fn verify(
+        self,
+        verifying_pk: &PublicKey,
+    ) -> Result<MetadataResponsePayload, VerificationError> {
         if self
             .signature
             .verify(verifying_pk, &self.payload.to_bytes())
         {
-            Some(self.payload.clone())
+            Ok(self.payload)
         } else {
-            None
+            Err(VerificationError)
         }
     }
 }
